using System;
using System.Drawing;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml;

namespace Glow
{
	public class SwarmmakerForm : Form
	{
		private Image background;
		private RectangleF clip;
		private Rectangle screenRectangle;
		private double angleStart;
		private int currentFrameNumber = int.MaxValue;
		// Kulturunabhängiges (invariantes) Zahlenformat zum Speichern verwenden.
		private System.Globalization.NumberFormatInfo format =
			System.Globalization.CultureInfo.InvariantCulture.NumberFormat;

		/// <summary>
		/// Erforderliche Designervariable.
		/// </summary>
		private System.ComponentModel.Container components = null;
		private System.Windows.Forms.NumericUpDown numericUpDownSinX;
		private System.Windows.Forms.NumericUpDown numericUpDownCosX;
		private System.Windows.Forms.NumericUpDown numericUpDownSinY;
		private System.Windows.Forms.NumericUpDown numericUpDownCosY;
		private System.Windows.Forms.Panel panelSettings;
		private System.Windows.Forms.Label labelTrail;
		private System.Windows.Forms.NumericUpDown numericUpDownTrail;
		private System.Windows.Forms.Label labelThickness;
		private System.Windows.Forms.NumericUpDown numericUpDownThickness;
		private System.Windows.Forms.Label labelSize;
		private System.Windows.Forms.NumericUpDown numericUpDownSize;
		private System.Windows.Forms.NumericUpDown numericUpDownCount;
		private System.Windows.Forms.Label labelCount;
		private System.Windows.Forms.Label label8;
		private System.Windows.Forms.Label label7;
		private System.Windows.Forms.Label label6;
		private System.Windows.Forms.Label label5;
		private System.Windows.Forms.NumericUpDown numericUpDownBitmapSize;
		private System.Windows.Forms.Label labelBitmapSize;
		private System.Windows.Forms.NumericUpDown numericUpDownFrames;
		private System.Windows.Forms.Label labelFrames;
		private System.Windows.Forms.NumericUpDown numericUpDownSpeed;
		private System.Windows.Forms.Label labelSpeed;
		private System.Windows.Forms.StatusBar statusBar;
		private System.Windows.Forms.CheckBox checkBoxBorder;
		private System.Windows.Forms.Button buttonLoad;
		private System.Windows.Forms.Button buttonSave;
		private System.Windows.Forms.Button buttonColor;
		private System.Windows.Forms.Button buttonScale;
		private System.Windows.Forms.Timer timerRefresh;
		private System.Windows.Forms.ColorDialog colorDialog;
		private System.Windows.Forms.OpenFileDialog openFileDialog;
		private System.Windows.Forms.SaveFileDialog saveFileDialog;

		public SwarmmakerForm()
		{
			InitializeComponent();

			// Double-Buffering verzögert das Zeichnen ein wenig, dafür flackert es nicht.
			SetStyle(ControlStyles.AllPaintingInWmPaint, true);
			SetStyle(ControlStyles.DoubleBuffer,         true);
			SetStyle(ControlStyles.UserPaint,            true);

			string backgroundFilename = "Background.png";
			if (System.IO.File.Exists(backgroundFilename))
				background = Bitmap.FromFile(backgroundFilename);

			// Nicht sehr elegant, aber diese Variante ist kompatibel zum Compact Framework.
			openFileDialog.InitialDirectory = System.IO.Path.GetDirectoryName(
				System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase).
				Replace(@"file:\", "");
			saveFileDialog.InitialDirectory = openFileDialog.InitialDirectory;
		}

		/// <summary>
		/// Die verwendeten Ressourcen bereinigen.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Vom Windows Form-Designer generierter Code
		/// <summary>
		/// Erforderliche Methode für die Designerunterstützung.
		/// Der Inhalt der Methode darf nicht mit dem Code-Editor geändert werden.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(SwarmmakerForm));
			this.timerRefresh = new System.Windows.Forms.Timer(this.components);
			this.numericUpDownSinX = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownCosX = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownSinY = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownCosY = new System.Windows.Forms.NumericUpDown();
			this.panelSettings = new System.Windows.Forms.Panel();
			this.buttonColor = new System.Windows.Forms.Button();
			this.buttonLoad = new System.Windows.Forms.Button();
			this.buttonSave = new System.Windows.Forms.Button();
			this.numericUpDownSpeed = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownFrames = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownBitmapSize = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownCount = new System.Windows.Forms.NumericUpDown();
			this.buttonScale = new System.Windows.Forms.Button();
			this.numericUpDownThickness = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownSize = new System.Windows.Forms.NumericUpDown();
			this.numericUpDownTrail = new System.Windows.Forms.NumericUpDown();
			this.labelCount = new System.Windows.Forms.Label();
			this.labelTrail = new System.Windows.Forms.Label();
			this.labelSize = new System.Windows.Forms.Label();
			this.label8 = new System.Windows.Forms.Label();
			this.label7 = new System.Windows.Forms.Label();
			this.label6 = new System.Windows.Forms.Label();
			this.label5 = new System.Windows.Forms.Label();
			this.labelSpeed = new System.Windows.Forms.Label();
			this.labelFrames = new System.Windows.Forms.Label();
			this.labelBitmapSize = new System.Windows.Forms.Label();
			this.labelThickness = new System.Windows.Forms.Label();
			this.checkBoxBorder = new System.Windows.Forms.CheckBox();
			this.saveFileDialog = new System.Windows.Forms.SaveFileDialog();
			this.openFileDialog = new System.Windows.Forms.OpenFileDialog();
			this.statusBar = new System.Windows.Forms.StatusBar();
			this.colorDialog = new System.Windows.Forms.ColorDialog();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSinX)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCosX)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSinY)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCosY)).BeginInit();
			this.panelSettings.SuspendLayout();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSpeed)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownFrames)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBitmapSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCount)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownThickness)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSize)).BeginInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownTrail)).BeginInit();
			this.SuspendLayout();
			// 
			// timerRefresh
			// 
			this.timerRefresh.Enabled = true;
			this.timerRefresh.Interval = 40;
			this.timerRefresh.Tick += new System.EventHandler(this.timerRefresh_Tick);
			// 
			// numericUpDownSinX
			// 
			this.numericUpDownSinX.Location = new System.Drawing.Point(446, 12);
			this.numericUpDownSinX.Minimum = new System.Decimal(new int[] {
																			  100,
																			  0,
																			  0,
																			  -2147483648});
			this.numericUpDownSinX.Name = "numericUpDownSinX";
			this.numericUpDownSinX.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownSinX.TabIndex = 15;
			this.numericUpDownSinX.Value = new System.Decimal(new int[] {
																			5,
																			0,
																			0,
																			0});
			// 
			// numericUpDownCosX
			// 
			this.numericUpDownCosX.Location = new System.Drawing.Point(446, 38);
			this.numericUpDownCosX.Minimum = new System.Decimal(new int[] {
																			  100,
																			  0,
																			  0,
																			  -2147483648});
			this.numericUpDownCosX.Name = "numericUpDownCosX";
			this.numericUpDownCosX.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownCosX.TabIndex = 17;
			this.numericUpDownCosX.Value = new System.Decimal(new int[] {
																			2,
																			0,
																			0,
																			0});
			// 
			// numericUpDownSinY
			// 
			this.numericUpDownSinY.Location = new System.Drawing.Point(446, 64);
			this.numericUpDownSinY.Minimum = new System.Decimal(new int[] {
																			  100,
																			  0,
																			  0,
																			  -2147483648});
			this.numericUpDownSinY.Name = "numericUpDownSinY";
			this.numericUpDownSinY.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownSinY.TabIndex = 19;
			this.numericUpDownSinY.Value = new System.Decimal(new int[] {
																			1,
																			0,
																			0,
																			0});
			// 
			// numericUpDownCosY
			// 
			this.numericUpDownCosY.Location = new System.Drawing.Point(446, 90);
			this.numericUpDownCosY.Minimum = new System.Decimal(new int[] {
																			  100,
																			  0,
																			  0,
																			  -2147483648});
			this.numericUpDownCosY.Name = "numericUpDownCosY";
			this.numericUpDownCosY.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownCosY.TabIndex = 21;
			this.numericUpDownCosY.Value = new System.Decimal(new int[] {
																			7,
																			0,
																			0,
																			0});
			// 
			// panelSettings
			// 
			this.panelSettings.Controls.Add(this.buttonColor);
			this.panelSettings.Controls.Add(this.buttonLoad);
			this.panelSettings.Controls.Add(this.buttonSave);
			this.panelSettings.Controls.Add(this.numericUpDownSpeed);
			this.panelSettings.Controls.Add(this.numericUpDownFrames);
			this.panelSettings.Controls.Add(this.numericUpDownBitmapSize);
			this.panelSettings.Controls.Add(this.numericUpDownCount);
			this.panelSettings.Controls.Add(this.buttonScale);
			this.panelSettings.Controls.Add(this.numericUpDownThickness);
			this.panelSettings.Controls.Add(this.numericUpDownSize);
			this.panelSettings.Controls.Add(this.numericUpDownTrail);
			this.panelSettings.Controls.Add(this.numericUpDownSinX);
			this.panelSettings.Controls.Add(this.numericUpDownCosX);
			this.panelSettings.Controls.Add(this.numericUpDownSinY);
			this.panelSettings.Controls.Add(this.numericUpDownCosY);
			this.panelSettings.Controls.Add(this.labelCount);
			this.panelSettings.Controls.Add(this.labelTrail);
			this.panelSettings.Controls.Add(this.labelSize);
			this.panelSettings.Controls.Add(this.label8);
			this.panelSettings.Controls.Add(this.label7);
			this.panelSettings.Controls.Add(this.label6);
			this.panelSettings.Controls.Add(this.label5);
			this.panelSettings.Controls.Add(this.labelSpeed);
			this.panelSettings.Controls.Add(this.labelFrames);
			this.panelSettings.Controls.Add(this.labelBitmapSize);
			this.panelSettings.Controls.Add(this.labelThickness);
			this.panelSettings.Controls.Add(this.checkBoxBorder);
			this.panelSettings.Dock = System.Windows.Forms.DockStyle.Top;
			this.panelSettings.Location = new System.Drawing.Point(0, 0);
			this.panelSettings.Name = "panelSettings";
			this.panelSettings.Size = new System.Drawing.Size(664, 122);
			this.panelSettings.TabIndex = 0;
			// 
			// buttonColor
			// 
			this.buttonColor.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.buttonColor.Location = new System.Drawing.Point(512, 62);
			this.buttonColor.Name = "buttonColor";
			this.buttonColor.Size = new System.Drawing.Size(48, 23);
			this.buttonColor.TabIndex = 26;
			this.buttonColor.Text = "&Color";
			this.buttonColor.Click += new System.EventHandler(this.buttonColor_Click);
			// 
			// buttonLoad
			// 
			this.buttonLoad.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.buttonLoad.Location = new System.Drawing.Point(512, 10);
			this.buttonLoad.Name = "buttonLoad";
			this.buttonLoad.Size = new System.Drawing.Size(140, 23);
			this.buttonLoad.TabIndex = 22;
			this.buttonLoad.Text = "&Load... (F3)";
			this.buttonLoad.Click += new System.EventHandler(this.buttonLoad_Click);
			// 
			// buttonSave
			// 
			this.buttonSave.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.buttonSave.Location = new System.Drawing.Point(512, 36);
			this.buttonSave.Name = "buttonSave";
			this.buttonSave.Size = new System.Drawing.Size(140, 23);
			this.buttonSave.TabIndex = 23;
			this.buttonSave.Text = "&Save as... (F2)";
			this.buttonSave.Click += new System.EventHandler(this.buttonSave_Click);
			// 
			// numericUpDownSpeed
			// 
			this.numericUpDownSpeed.Location = new System.Drawing.Point(136, 90);
			this.numericUpDownSpeed.Maximum = new System.Decimal(new int[] {
																			   60,
																			   0,
																			   0,
																			   0});
			this.numericUpDownSpeed.Minimum = new System.Decimal(new int[] {
																			   1,
																			   0,
																			   0,
																			   0});
			this.numericUpDownSpeed.Name = "numericUpDownSpeed";
			this.numericUpDownSpeed.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownSpeed.TabIndex = 5;
			this.numericUpDownSpeed.Value = new System.Decimal(new int[] {
																			 30,
																			 0,
																			 0,
																			 0});
			this.numericUpDownSpeed.ValueChanged += new System.EventHandler(this.numericUpDownSpeed_ValueChanged);
			// 
			// numericUpDownFrames
			// 
			this.numericUpDownFrames.Location = new System.Drawing.Point(136, 38);
			this.numericUpDownFrames.Maximum = new System.Decimal(new int[] {
																				3600,
																				0,
																				0,
																				0});
			this.numericUpDownFrames.Minimum = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				0});
			this.numericUpDownFrames.Name = "numericUpDownFrames";
			this.numericUpDownFrames.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownFrames.TabIndex = 3;
			this.numericUpDownFrames.Value = new System.Decimal(new int[] {
																			  30,
																			  0,
																			  0,
																			  0});
			// 
			// numericUpDownBitmapSize
			// 
			this.numericUpDownBitmapSize.Increment = new System.Decimal(new int[] {
																					  32,
																					  0,
																					  0,
																					  0});
			this.numericUpDownBitmapSize.Location = new System.Drawing.Point(136, 12);
			this.numericUpDownBitmapSize.Maximum = new System.Decimal(new int[] {
																					1024,
																					0,
																					0,
																					0});
			this.numericUpDownBitmapSize.Minimum = new System.Decimal(new int[] {
																					32,
																					0,
																					0,
																					0});
			this.numericUpDownBitmapSize.Name = "numericUpDownBitmapSize";
			this.numericUpDownBitmapSize.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownBitmapSize.TabIndex = 1;
			this.numericUpDownBitmapSize.Value = new System.Decimal(new int[] {
																				  256,
																				  0,
																				  0,
																				  0});
			this.numericUpDownBitmapSize.ValueChanged += new System.EventHandler(this.buttonScale_Click);
			// 
			// numericUpDownCount
			// 
			this.numericUpDownCount.Location = new System.Drawing.Point(283, 12);
			this.numericUpDownCount.Maximum = new System.Decimal(new int[] {
																			   10000,
																			   0,
																			   0,
																			   0});
			this.numericUpDownCount.Minimum = new System.Decimal(new int[] {
																			   1,
																			   0,
																			   0,
																			   0});
			this.numericUpDownCount.Name = "numericUpDownCount";
			this.numericUpDownCount.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownCount.TabIndex = 7;
			this.numericUpDownCount.Value = new System.Decimal(new int[] {
																			 15,
																			 0,
																			 0,
																			 0});
			// 
			// buttonScale
			// 
			this.buttonScale.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right)));
			this.buttonScale.Location = new System.Drawing.Point(512, 88);
			this.buttonScale.Name = "buttonScale";
			this.buttonScale.Size = new System.Drawing.Size(140, 23);
			this.buttonScale.TabIndex = 25;
			this.buttonScale.Text = "&Fit to bitmap size (F5)";
			this.buttonScale.Click += new System.EventHandler(this.buttonScale_Click);
			// 
			// numericUpDownThickness
			// 
			this.numericUpDownThickness.Location = new System.Drawing.Point(283, 64);
			this.numericUpDownThickness.Name = "numericUpDownThickness";
			this.numericUpDownThickness.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownThickness.TabIndex = 11;
			this.numericUpDownThickness.Value = new System.Decimal(new int[] {
																				 1,
																				 0,
																				 0,
																				 0});
			// 
			// numericUpDownSize
			// 
			this.numericUpDownSize.DecimalPlaces = 1;
			this.numericUpDownSize.Increment = new System.Decimal(new int[] {
																				1,
																				0,
																				0,
																				65536});
			this.numericUpDownSize.Location = new System.Drawing.Point(283, 38);
			this.numericUpDownSize.Name = "numericUpDownSize";
			this.numericUpDownSize.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownSize.TabIndex = 9;
			this.numericUpDownSize.Value = new System.Decimal(new int[] {
																			2,
																			0,
																			0,
																			0});
			// 
			// numericUpDownTrail
			// 
			this.numericUpDownTrail.DecimalPlaces = 1;
			this.numericUpDownTrail.Increment = new System.Decimal(new int[] {
																				 1,
																				 0,
																				 0,
																				 65536});
			this.numericUpDownTrail.Location = new System.Drawing.Point(283, 90);
			this.numericUpDownTrail.Maximum = new System.Decimal(new int[] {
																			   1000,
																			   0,
																			   0,
																			   0});
			this.numericUpDownTrail.Minimum = new System.Decimal(new int[] {
																			   1000,
																			   0,
																			   0,
																			   -2147483648});
			this.numericUpDownTrail.Name = "numericUpDownTrail";
			this.numericUpDownTrail.Size = new System.Drawing.Size(60, 20);
			this.numericUpDownTrail.TabIndex = 13;
			this.numericUpDownTrail.Value = new System.Decimal(new int[] {
																			 8,
																			 0,
																			 0,
																			 65536});
			// 
			// labelCount
			// 
			this.labelCount.Location = new System.Drawing.Point(202, 14);
			this.labelCount.Name = "labelCount";
			this.labelCount.Size = new System.Drawing.Size(81, 16);
			this.labelCount.TabIndex = 6;
			this.labelCount.Text = "Particle &count:";
			this.labelCount.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelTrail
			// 
			this.labelTrail.Location = new System.Drawing.Point(202, 92);
			this.labelTrail.Name = "labelTrail";
			this.labelTrail.Size = new System.Drawing.Size(81, 16);
			this.labelTrail.TabIndex = 12;
			this.labelTrail.Text = "&Trail size:";
			this.labelTrail.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelSize
			// 
			this.labelSize.Location = new System.Drawing.Point(202, 40);
			this.labelSize.Name = "labelSize";
			this.labelSize.Size = new System.Drawing.Size(81, 16);
			this.labelSize.TabIndex = 8;
			this.labelSize.Text = "Particle size:";
			this.labelSize.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label8
			// 
			this.label8.Location = new System.Drawing.Point(349, 92);
			this.label8.Name = "label8";
			this.label8.Size = new System.Drawing.Size(97, 16);
			this.label8.TabIndex = 20;
			this.label8.Text = "Vertical cosine:";
			this.label8.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label7
			// 
			this.label7.Location = new System.Drawing.Point(349, 66);
			this.label7.Name = "label7";
			this.label7.Size = new System.Drawing.Size(97, 16);
			this.label7.TabIndex = 18;
			this.label7.Text = "&Vertical sine:";
			this.label7.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label6
			// 
			this.label6.Location = new System.Drawing.Point(349, 40);
			this.label6.Name = "label6";
			this.label6.Size = new System.Drawing.Size(97, 16);
			this.label6.TabIndex = 16;
			this.label6.Text = "Horizontal cosine:";
			this.label6.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// label5
			// 
			this.label5.Location = new System.Drawing.Point(349, 14);
			this.label5.Name = "label5";
			this.label5.Size = new System.Drawing.Size(97, 16);
			this.label5.TabIndex = 14;
			this.label5.Text = "&Horizontal sine:";
			this.label5.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelSpeed
			// 
			this.labelSpeed.Location = new System.Drawing.Point(12, 92);
			this.labelSpeed.Name = "labelSpeed";
			this.labelSpeed.Size = new System.Drawing.Size(124, 16);
			this.labelSpeed.TabIndex = 4;
			this.labelSpeed.Text = "Frames per second:";
			this.labelSpeed.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelFrames
			// 
			this.labelFrames.Location = new System.Drawing.Point(12, 40);
			this.labelFrames.Name = "labelFrames";
			this.labelFrames.Size = new System.Drawing.Size(124, 16);
			this.labelFrames.TabIndex = 2;
			this.labelFrames.Text = "Total number of frames:";
			this.labelFrames.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelBitmapSize
			// 
			this.labelBitmapSize.Location = new System.Drawing.Point(12, 14);
			this.labelBitmapSize.Name = "labelBitmapSize";
			this.labelBitmapSize.Size = new System.Drawing.Size(124, 16);
			this.labelBitmapSize.TabIndex = 0;
			this.labelBitmapSize.Text = "Bitmap size:";
			this.labelBitmapSize.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// labelThickness
			// 
			this.labelThickness.Location = new System.Drawing.Point(12, 66);
			this.labelThickness.Name = "labelThickness";
			this.labelThickness.Size = new System.Drawing.Size(271, 16);
			this.labelThickness.TabIndex = 10;
			this.labelThickness.Text = "Particle thickness:";
			this.labelThickness.TextAlign = System.Drawing.ContentAlignment.TopRight;
			// 
			// checkBoxBorder
			// 
			this.checkBoxBorder.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.checkBoxBorder.Location = new System.Drawing.Point(566, 65);
			this.checkBoxBorder.Name = "checkBoxBorder";
			this.checkBoxBorder.Size = new System.Drawing.Size(96, 17);
			this.checkBoxBorder.TabIndex = 24;
			this.checkBoxBorder.Text = "Show border";
			// 
			// saveFileDialog
			// 
			this.saveFileDialog.Filter = "Swarmmaker settings (*.xml)|*.xml|PNG image sequence (*.png)|*.png";
			this.saveFileDialog.Title = "Save as...";
			// 
			// openFileDialog
			// 
			this.openFileDialog.Filter = "Swarmmaker settings (*.xml)|*.xml";
			this.openFileDialog.Title = "Load...";
			// 
			// statusBar
			// 
			this.statusBar.Location = new System.Drawing.Point(0, 444);
			this.statusBar.Name = "statusBar";
			this.statusBar.Size = new System.Drawing.Size(664, 22);
			this.statusBar.TabIndex = 1;
			this.statusBar.Text = "Ready";
			// 
			// colorDialog
			// 
			this.colorDialog.AnyColor = true;
			this.colorDialog.Color = System.Drawing.Color.White;
			this.colorDialog.FullOpen = true;
			// 
			// SwarmmakerForm
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(664, 466);
			this.Controls.Add(this.statusBar);
			this.Controls.Add(this.panelSettings);
			this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
			this.KeyPreview = true;
			this.MinimumSize = new System.Drawing.Size(672, 300);
			this.Name = "SwarmmakerForm";
			this.Text = "Glow Swarmmaker 0.2 alpha";
			this.KeyDown += new System.Windows.Forms.KeyEventHandler(this.SwarmmakerForm_KeyDown);
			this.Resize += new System.EventHandler(this.SwarmmakerForm_Resize);
			this.Load += new System.EventHandler(this.SwarmmakerForm_Load);
			this.Paint += new System.Windows.Forms.PaintEventHandler(this.SwarmmakerForm_Paint);
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSinX)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCosX)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSinY)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCosY)).EndInit();
			this.panelSettings.ResumeLayout(false);
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSpeed)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownFrames)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownBitmapSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownCount)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownThickness)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownSize)).EndInit();
			((System.ComponentModel.ISupportInitialize)(this.numericUpDownTrail)).EndInit();
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// Der Haupteinstiegspunkt für die Anwendung.
		/// </summary>
		[STAThread]
		static void Main()
		{
			Application.EnableVisualStyles();
			Application.Run(new SwarmmakerForm());
		}

		private void SwarmmakerForm_Load(object sender, EventArgs e)
		{
			SwarmmakerForm_Resize(sender, e);
			buttonScale_Click(sender, e);
			numericUpDownSpeed_ValueChanged(sender, e);
		}

		private void SwarmmakerForm_Resize(object sender, EventArgs e)
		{
			screenRectangle = new Rectangle(panelSettings.Left, panelSettings.Bottom,
				panelSettings.Width, statusBar.Top - panelSettings.Bottom);
		}

		private void SwarmmakerForm_KeyDown(object sender, KeyEventArgs e)
		{
			switch (e.KeyCode)
			{
				case Keys.F2:
					buttonSave_Click(sender, e);
					break;
				case Keys.F3:
					buttonLoad_Click(sender, e);
					break;
				case Keys.F5:
					buttonScale_Click(sender, e);
					break;
				case Keys.F6:
					Random r = new Random();
					foreach (Control control in panelSettings.Controls)
					{
						if (control is NumericUpDown && control != numericUpDownBitmapSize && control != numericUpDownFrames && control != numericUpDownSpeed)
						{
							NumericUpDown c = control as NumericUpDown;
							Decimal range = (c.Maximum - c.Minimum) / 1000;
							Decimal v = c.Value + (decimal)r.Next() / (decimal)int.MaxValue * range - range / 2;
							c.Value = Math.Min(Math.Max(c.Minimum, v), c.Maximum);
						}
					}
					break;
				case Keys.F7:
					if (ActiveControl is NumericUpDown)
					{
						NumericUpDown c = ActiveControl as NumericUpDown;
						c.Value = c.Minimum + (decimal)new Random().Next() / (decimal)int.MaxValue * (c.Maximum - c.Minimum);
					}
					break;
				default:
					return;
			}
			e.Handled = true;
		}

		private void SwarmmakerForm_Paint(object sender, PaintEventArgs e)
		{
			PaintSwarm(e.Graphics);
		}

		private void timerRefresh_Tick(object sender, EventArgs e)
		{
			Invalidate();
		}

		private void numericUpDownSpeed_ValueChanged(object sender, EventArgs e)
		{
			timerRefresh.Interval = (int)Math.Round(1000 / numericUpDownSpeed.Value);
		}

		private void buttonColor_Click(object sender, EventArgs e)
		{
			colorDialog.ShowDialog();
		}

		private void buttonScale_Click(object sender, EventArgs e)
		{
			clip = new RectangleF(int.MaxValue, int.MaxValue, int.MinValue, int.MinValue);
		}

		private void buttonLoad_Click(object sender, EventArgs e)
		{
			if (openFileDialog.ShowDialog() != DialogResult.OK)
				return;

			if (!LoadSettings(openFileDialog.FileName))
			{
				MessageBox.Show("Not a valid settings file.",
					"Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
			}
			buttonScale_Click(sender, e);
			saveFileDialog.FileName = openFileDialog.FileName;
		}

		private void buttonSave_Click(object sender, EventArgs e)
		{
			// Datenendung verwerfen, damit man problemlos das Dateiformat wechseln kann.
			saveFileDialog.FileName = new Regex(@"\.[^.]*$",
				RegexOptions.IgnoreCase).Replace(saveFileDialog.FileName, "");
			if (saveFileDialog.ShowDialog() != DialogResult.OK)
				return;

			// Die eingegebene Dateiendung wird praktisch ignoriert, relevant ist das im
			// Filter ausgewählte Dateiformat.
			string[] filter = saveFileDialog.Filter.Split("|".ToCharArray());
			int index = saveFileDialog.FilterIndex * 2 - 1;
			if (index >= filter.Length)
			{
				MessageBox.Show("Invalid file format.", "Error", MessageBoxButtons.OK,
					MessageBoxIcon.Error);
				return;
			}

			switch (filter[index].Substring(2, 3).ToLower())
			{
				case "png":
					if (!saveFileDialog.FileName.ToLower().EndsWith(".png"))
						saveFileDialog.FileName += ".png";
					statusBar.Text     = "Saving image sequence, please wait...";
					angleStart         = 0;
					currentFrameNumber = -(int)numericUpDownFrames.Value;
					buttonScale_Click(sender, e);
					break;
				default:
					if (!saveFileDialog.FileName.ToLower().EndsWith(".xml"))
						saveFileDialog.FileName += ".xml";
					SaveSettings(saveFileDialog.FileName);
					openFileDialog.FileName = saveFileDialog.FileName;
					break;
			}
		}

		private bool LoadSettings(string filename)
		{
			XmlDocument doc = new XmlDocument();
			try
			{
				doc.Load(filename);

				XmlNodeList window = doc.GetElementsByTagName("Window");
				if (window.Count > 0)
				{
					Width  = Convert.ToInt32(window[0].Attributes["width"].Value);
					Height = Convert.ToInt32(window[0].Attributes["height"].Value);
				}

				XmlNode node = doc.SelectSingleNode("//Setting[@name='Color']/@value");
				if (node != null)
				{
					colorDialog.Color = Color.FromArgb(Int32.Parse(node.Value,
						System.Globalization.NumberStyles.AllowHexSpecifier));
				}
				else
					colorDialog.Color = Color.White;

				foreach (Control control in panelSettings.Controls)
				{
					node = doc.SelectSingleNode("//Setting[@name='" + control.Name + "']/@value");
					if (node == null || node.NodeType != XmlNodeType.Attribute)
						continue;
					if (control is NumericUpDown)
						(control as NumericUpDown).Value = Convert.ToDecimal(node.Value, format);
					else if (control is ComboBox)
						(control as ComboBox).SelectedIndex = Convert.ToInt32(node.Value, format);
				}
			}
			catch
			{
				return false;
			}
			return true;
		}

		private void SaveSettings(string filename)
		{
			XmlDocument doc        = new XmlDocument();
			XmlNode root           = doc.CreateElement("SwarmmakerSettings");

			XmlNode node           = doc.CreateElement("Window");
			XmlAttribute attribute = doc.CreateAttribute("width");
			attribute.Value        = Width.ToString(format);
			node.Attributes.Append(attribute);
			attribute              = doc.CreateAttribute("height");
			attribute.Value        = Height.ToString(format);
			node.Attributes.Append(attribute);
			root.AppendChild(node);

			node                   = doc.CreateElement("Setting");
			attribute              = doc.CreateAttribute("name");
			attribute.Value        = "Color";
			node.Attributes.Append(attribute);
			attribute              = doc.CreateAttribute("value");
			attribute.Value        = colorDialog.Color.ToArgb().ToString("X");
			node.Attributes.Append(attribute);
			root.AppendChild(node);

			foreach (Control control in panelSettings.Controls)
			{
				string value;
				if (control is ComboBox)
					value = (control as ComboBox).SelectedIndex.ToString(format);
				else if (control is NumericUpDown)
					value = (control as NumericUpDown).Value.ToString(format);
				else
					continue;
				node            = doc.CreateElement("Setting");
				attribute       = doc.CreateAttribute("name");
				attribute.Value = control.Name;
				node.Attributes.Append(attribute);
				attribute       = doc.CreateAttribute("value");
				attribute.Value = value;
				node.Attributes.Append(attribute);
				root.AppendChild(node);
			}
			doc.AppendChild(root);
			doc.Save(filename);
		}

		private void PaintSwarm(Graphics graphicsScreen)
		{
			Pen pen     = new Pen(colorDialog.Color);
			Brush brush = new SolidBrush(colorDialog.Color);

			int bitmapSize  = (int)numericUpDownBitmapSize.Value;
			int radius      = bitmapSize / 2;
			int frames      = (int)numericUpDownFrames.Value;
			int count       = (int)numericUpDownCount.Value;
			double distance = 2.0 * Math.PI / (double)frames / (double)count;
			double trail    = -distance * (double)numericUpDownTrail.Value;
			float size      = (float)numericUpDownSize.Value;
			int thickness   = (int)numericUpDownThickness.Value;
			double sinX     = (double)numericUpDownSinX.Value;
			double cosX     = (double)numericUpDownCosX.Value;
			double sinY     = (double)numericUpDownSinY.Value;
			double cosY     = (double)numericUpDownCosY.Value;

			Image bitmap = new Bitmap(bitmapSize, bitmapSize);
			Graphics g = Graphics.FromImage(bitmap);
			if (checkBoxBorder.Checked)
				g.Clear(Color.Black);
			g.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.AntiAlias;

			for (double angleIncrement = 0; angleIncrement < 2.0 * Math.PI;
				angleIncrement += distance * frames)
			{
				double angle = angleStart + angleIncrement;

				double x = radius +
					Math.Sin(angle * sinX) * radius * 0.5 +
					Math.Cos(angle * cosX) * radius * 0.5;
				double y = radius +
					Math.Sin(angle * sinY) * radius * 0.5 +
					Math.Cos(angle * cosY) * radius * 0.5;
				double x0 = radius +
					Math.Sin((angle + trail) * sinX) * radius * 0.5 +
					Math.Cos((angle + trail) * cosX) * radius * 0.5;
				double y0 = radius +
					Math.Sin((angle + trail) * sinY) * radius * 0.5 +
					Math.Cos((angle + trail) * cosY) * radius * 0.5;

				// Calculate perfect scale.
				clip.X      = (float)Math.Min(clip.X,      Math.Floor(x));
				clip.Y      = (float)Math.Min(clip.Y,      Math.Floor(y));
				clip.Width  = (float)Math.Max(clip.Width,  Math.Ceiling(x) - clip.X + size);
				clip.Height = (float)Math.Max(clip.Height, Math.Ceiling(y) - clip.Y + size);
				float scaleX = bitmap.Width  / clip.Width;
				float scaleY = bitmap.Height / clip.Height;

				float x1 = ((float)x - clip.X) * scaleX;
				float y1 = ((float)y - clip.Y) * scaleY;

				// Draw trail.
				g.DrawLine(pen, x1 + size * 0.5f, y1 + size * 0.5f,
					((float)x0 - clip.X) * scaleX + size * 0.5f,
					((float)y0 - clip.Y) * scaleY + size * 0.5f);
				// Draw circle.
				for (int i = 0; i < thickness; i++)
					g.FillEllipse(brush, x1, y1, size, size);
			}
			angleStart += distance;
			if (angleStart >= 2.0 * Math.PI)
				angleStart = 0.0;

			// Paint to screen.
			if (background != null)
			{
				graphicsScreen.DrawImageUnscaled(background,
					screenRectangle.X + (screenRectangle.Width  - background.Width)  / 2,
					screenRectangle.Y + (screenRectangle.Height - background.Height) / 2);
			}
			else if (checkBoxBorder.Checked)
				graphicsScreen.Clear(Color.Gray);
			else
				graphicsScreen.Clear(Color.Black);
			int screenX = screenRectangle.X + (screenRectangle.Width  - bitmap.Width)  / 2;
			int screenY = screenRectangle.Y + (screenRectangle.Height - bitmap.Height) / 2;
			graphicsScreen.DrawImageUnscaled(bitmap, screenX, screenY);

			// Save to disk.
			if (currentFrameNumber >= 0 && currentFrameNumber < frames)
			{
				string filename = new Regex(@"(?:_\d{3})?(?=\.[^.]*$)").Replace(
					saveFileDialog.FileName,
					String.Format("_{0:d3}", 1 + currentFrameNumber));
				try
				{
					bitmap.Save(filename, System.Drawing.Imaging.ImageFormat.Png);
				}
				catch (Exception ex)
				{
					MessageBox.Show(ex.Message);
					currentFrameNumber = int.MaxValue;
				}
				statusBar.Text = String.Format("File \"{0}\" was saved successfully",
					new System.IO.FileInfo(filename).Name);
			}
			if (currentFrameNumber < int.MaxValue)
				currentFrameNumber++;
		}
	}
}